from pathlib import Path
import sys

from models import LoadCheckpoint
import torch
import numpy as np
import cv2
from PIL import Image
from datetime import datetime
import argparse 
class ImageDemoOptions():
    def __init__(self):
        self.parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
        self.parser.add_argument('--net_model_path', '-n', type=str, default="../models/last_transfered_net.pth", help="Net model path folder")
        self.parser.add_argument('--lut_model_path', '-l', type=str, default="../models/last_transfered_lut.pth", help="Lut model path folder")
        self.parser.add_argument('--hr_image_path', '-a', type=str, default="../data/Set14/HR/monarch.png", help="HR image path")
        self.parser.add_argument('--lr_image_path', '-b', type=str, default="../data/Set14/LR/X4/monarch.png", help="LR image path")
        self.parser.add_argument('--project_path', type=str, default="../", help="Project path.")  
        self.parser.add_argument('--batch_size', type=int, default=2**10, help="Size of the batch for the input domain values.")  

    def parse_args(self): 
        args = self.parser.parse_args()       
        args.project_path = Path(args.project_path).resolve() 
        args.hr_image_path = Path(args.hr_image_path).resolve()
        args.lr_image_path = Path(args.lr_image_path).resolve()
        args.net_model_path = Path(args.net_model_path).resolve()
        args.lut_model_path = Path(args.lut_model_path).resolve()         
        return args

    def __repr__(self):
        config = self.parser.parse_args() 
        message = ''
        message += '----------------- Options ---------------\n'
        for k, v in sorted(vars(config).items()):
            comment = ''
            default = self.parser.get_default(k)
            if v != default:
                comment = '\t[default: %s]' % str(default)
            message += '{:>25}: {:<30}{}\n'.format(str(k), str(v), comment)
        message += '----------------- End -------------------'
        return message

config_inst = ImageDemoOptions()
config = config_inst.parse_args()

start_script_time = datetime.now()

net_model = LoadCheckpoint(config.net_model_path).cuda()
lut_model = LoadCheckpoint(config.lut_model_path).cuda()

print(net_model)
print(lut_model)

lr_image = cv2.imread(str(config.lr_image_path))[:,:,::-1].copy()
image_gt = cv2.imread(str(config.hr_image_path))[:,:,::-1].copy()

input_image = torch.tensor(lr_image).type(torch.float32).permute(2,0,1)[None,...].cuda()

with torch.inference_mode():
    net_prediction = net_model(input_image).cpu().type(torch.uint8).squeeze().permute(1,2,0).numpy().copy()
    lut_prediction = lut_model(input_image).cpu().type(torch.uint8).squeeze().permute(1,2,0).numpy().copy()


image_gt = cv2.putText(image_gt, 'GT', org=(20, 50) , fontFace=cv2.FONT_HERSHEY_SIMPLEX, fontScale=1, color=(255,255,255), thickness=2, lineType= cv2.LINE_AA) 
image_net = cv2.putText(net_prediction, net_model.__class__.__name__, org=(20, 50) , fontFace=cv2.FONT_HERSHEY_SIMPLEX, fontScale=1, color=(255,255,255), thickness=2, lineType= cv2.LINE_AA) 
image_lut = cv2.putText(lut_prediction, lut_model.__class__.__name__, org=(20, 50) , fontFace=cv2.FONT_HERSHEY_SIMPLEX, fontScale=1, color=(255,255,255), thickness=2, lineType= cv2.LINE_AA) 

Image.fromarray(np.concatenate([image_gt, image_net, image_lut], 1)).save(config.project_path / "models" / 'last_transfered_demo.png')

print(datetime.now() - start_script_time )